@@ -1,45 +1,7 @@
 /*
-Planarity-Related Graph Algorithms Project
-Copyright (c) 1997-2010, John M. Boyer
-All rights reserved. Includes a reference implementation of the following:
-
-* John M. Boyer. "Simplified O(n) Algorithms for Planar Graph Embedding,
-  Kuratowski Subgraph Isolation, and Related Problems". Ph.D. Dissertation,
-  University of Victoria, 2001.
-
-* John M. Boyer and Wendy J. Myrvold. "On the Cutting Edge: Simplified O(n)
-  Planarity by Edge Addition". Journal of Graph Algorithms and Applications,
-  Vol. 8, No. 3, pp. 241-273, 2004.
-
-* John M. Boyer. "A New Method for Efficiently Generating Planar Graph
-  Visibility Representations". In P. Eades and P. Healy, editors,
-  Proceedings of the 13th International Conference on Graph Drawing 2005,
-  Lecture Notes Comput. Sci., Volume 3843, pp. 508-511, Springer-Verlag, 2006.
-
-Redistribution and use in source and binary forms, with or without modification,
-are permitted provided that the following conditions are met:
-
-* Redistributions of source code must retain the above copyright notice, this
-  list of conditions and the following disclaimer.
-
-* Redistributions in binary form must reproduce the above copyright notice, this
-  list of conditions and the following disclaimer in the documentation and/or
-  other materials provided with the distribution.
-
-* Neither the name of the Planarity-Related Graph Algorithms Project nor the names
-  of its contributors may be used to endorse or promote products derived from this
-  software without specific prior written permission.
-
-THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
-AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
-IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
-DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
-ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
-(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
-LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
-ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
-SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+Copyright (c) 1997-2022, John M. Boyer
+All rights reserved.
+See the LICENSE.TXT file for licensing information.
 */
 
 #define GRAPHTEST_C
@@ -47,11 +9,13 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include "graph.h"
 #include "stack.h"
 
+extern void _ClearVertexVisitedFlags(graphP theGraph, int);
+
 /* Private function declarations */
 
 int  _TestPath(graphP theGraph, int U, int V);
-int  _TryPath(graphP theGraph, int J, int V);
-void _MarkPath(graphP theGraph, int J);
+int  _TryPath(graphP theGraph, int e, int V);
+void _MarkPath(graphP theGraph, int e);
 int  _TestSubgraph(graphP theSubgraph, graphP theGraph);
 
 int  _CheckEmbeddingIntegrity(graphP theGraph, graphP origGraph);
@@ -176,15 +140,15 @@ int _CheckEmbeddingIntegrity(graphP theGraph, graphP origGraph)
  begins by placing all of the graph's arcs onto a stack and marking
  all of them as unvisited.  For each arc popped, if it is visited,
  it is immediately discarded and the next arc is popped.  Popping an
- unvisited arc J begins a face traversal.  We move to the true twin
- arc K of J, and obtain its successor arc L.  This amounts to
- always going clockwise or counterclockwise (depending on how the
- graph is drawn on the plane, or alternately whether one is above
- or below the plane).  This traversal continues until we make it
- back to J.  Each arc along the way is marked as visited.  Further,
- if L has been visited, then there is an error since an arc can only
- appear in one face (the twin arc appears in a separate face, which
- is traversed in the opposing direction).
+ unvisited arc e begins a face traversal.  We move to the true twin
+ arc of e, and obtain its successor arc.  This amounts to always
+ going clockwise or counterclockwise (depending on how the graph is
+ drawn on the plane, or alternately whether one is above or below
+ the plane).  This traversal continues until we make it back to the
+ original arc e. Each arc along the way is marked as visited.  Further,
+ if the successor arc has been visited, then there is an error since
+ an arc can only appear in one face (the twin arc appears in a separate
+ face, which is traversed in the opposing direction).
  If this algorithm succeeds without double visiting any arcs, and it
  produces the correct face count according to Euler's formula, then
  the embedding has all vertices oriented the same way.
@@ -197,7 +161,7 @@ int _CheckEmbeddingIntegrity(graphP theGraph, graphP origGraph)
 int  _CheckEmbeddingFacialIntegrity(graphP theGraph)
 {
 stackP theStack = theGraph->theStack;
-int I, e, J, JTwin, K, L, NumFaces, connectedComponents;
+int EsizeOccupied, v, e, eTwin, eStart, eNext, NumFaces, connectedComponents;
 
      if (theGraph == NULL)
          return NOTOK;
@@ -211,16 +175,18 @@ int I, e, J, JTwin, K, L, NumFaces, connectedComponents;
 
 /* Push all arcs and set them to unvisited */
 
-     for (e=0, J=theGraph->edgeOffset; e < theGraph->M + sp_GetCurrentSize(theGraph->edgeHoles); e++, J+=2)
+	 EsizeOccupied = gp_EdgeInUseIndexBound(theGraph);
+     for (e = gp_GetFirstEdge(theGraph); e < EsizeOccupied; e+=2)
      {
-          if (theGraph->G[J].v == NIL)
-              continue;
-
-          sp_Push(theStack, J);
-          theGraph->G[J].visited = 0;
-          JTwin = gp_GetTwinArc(theGraph, J);
-          sp_Push(theStack, JTwin);
-          theGraph->G[JTwin].visited = 0;
+    	  // Except skip edge holes
+          if (gp_EdgeInUse(theGraph, e))
+          {
+			  sp_Push(theStack, e);
+			  gp_ClearEdgeVisited(theGraph, e);
+			  eTwin = gp_GetTwinArc(theGraph, e);
+			  sp_Push(theStack, eTwin);
+			  gp_ClearEdgeVisited(theGraph, eTwin);
+          }
      }
 
      // There are M edges, so we better have pushed 2M arcs just now
@@ -236,20 +202,17 @@ int I, e, J, JTwin, K, L, NumFaces, connectedComponents;
      {
             /* Get an arc; if it has already been used by a face, then
                 don't use it to traverse a new face */
-            sp_Pop(theStack, J);
-            if (theGraph->G[J].visited) continue;
+            sp_Pop(theStack, eStart);
+            if (gp_GetEdgeVisited(theGraph, eStart)) continue;
 
-            L = NIL;
-            JTwin = J;
-            while (L != J)
-            {
-                K = gp_GetTwinArc(theGraph, JTwin);
-                L = gp_GetNextArcCircular(theGraph, K);
-                if (theGraph->G[L].visited)
+            e = eStart;
+            do {
+                eNext = gp_GetNextArcCircular(theGraph, gp_GetTwinArc(theGraph, e));
+                if (gp_GetEdgeVisited(theGraph, eNext))
                     return NOTOK;
-                theGraph->G[L].visited++;
-                JTwin = L;
-            }
+                gp_SetEdgeVisited(theGraph, eNext);
+                e = eNext;
+            } while (e != eStart);
             NumFaces++;
      }
 
@@ -258,13 +221,16 @@ int I, e, J, JTwin, K, L, NumFaces, connectedComponents;
     DFS parent, except in the case of isolated vertices, no face was counted
     so we do not subtract one. */
 
-     for (I=connectedComponents=0; I < theGraph->N; I++)
-          if (theGraph->V[I].DFSParent == NIL)
+     connectedComponents = 0;
+     for (v = gp_GetFirstVertex(theGraph); gp_VertexInRange(theGraph, v); v++)
+     {
+          if (gp_IsDFSTreeRoot(theGraph, v))
           {
-              if (gp_GetVertexDegree(theGraph, I) > 0)
+              if (gp_GetVertexDegree(theGraph, v) > 0)
                   NumFaces--;
               connectedComponents++;
           }
+     }
 
      NumFaces++;
 
@@ -290,27 +256,24 @@ int I, e, J, JTwin, K, L, NumFaces, connectedComponents;
 
 int _CheckAllVerticesOnExternalFace(graphP theGraph)
 {
-    int I;
+    int v;
 
     // Mark all vertices unvisited
-    for (I=0; I < theGraph->N; I++)
-        theGraph->G[I].visited = 0;
+    _ClearVertexVisitedFlags(theGraph, FALSE);
 
     // For each connected component, walk its external face and
     // mark the vertices as visited
-    for (I=0; I < theGraph->N; I++)
+    for (v = gp_GetFirstVertex(theGraph); gp_VertexInRange(theGraph, v); v++)
     {
-         if (theGraph->V[I].DFSParent == NIL)
-        	 _MarkExternalFaceVertices(theGraph, I);
+         if (gp_IsDFSTreeRoot(theGraph, v))
+        	 _MarkExternalFaceVertices(theGraph, v);
     }
 
     // If any vertex is unvisited, then the embedding is not an outerplanar
     // embedding, so we return NOTOK
-    for (I=0; I < theGraph->N; I++)
-        if (!theGraph->G[I].visited)
-        {
+    for (v = gp_GetFirstVertex(theGraph); gp_VertexInRange(theGraph, v); v++)
+        if (!gp_GetVertexVisited(theGraph, v))
             return NOTOK;
-        }
 
     // All vertices were found on external faces of the connected components
     // so the embedding is an outerplanar embedding and we return OK
@@ -334,22 +297,22 @@ int _CheckAllVerticesOnExternalFace(graphP theGraph)
 void _MarkExternalFaceVertices(graphP theGraph, int startVertex)
 {
     int nextVertex = startVertex;
-    int Jout = gp_GetFirstArc(theGraph, nextVertex);
-    int Jin;
+    int e = gp_GetFirstArc(theGraph, nextVertex);
+    int eTwin;
 
     // Handle the case of an isolated vertex
-    if (!gp_IsArc(theGraph, Jout))
+    if (gp_IsNotArc(e))
     {
-    	theGraph->G[startVertex].visited = 1;
+    	gp_SetVertexVisited(theGraph, startVertex);
     	return;
     }
 
     // Process a non-trivial connected component
     do {
-        theGraph->G[nextVertex].visited = 1;
+        gp_SetVertexVisited(theGraph, nextVertex);
 
         // The arc out of the vertex just visited points to the next vertex
-        nextVertex = theGraph->G[Jout].v;
+        nextVertex = gp_GetNeighbor(theGraph, e);
 
         // Arc used to enter the next vertex is needed so we can get the
         // next edge in rotation order.
@@ -358,7 +321,7 @@ void _MarkExternalFaceVertices(graphP theGraph, int startVertex)
         //       But _JoinBicomps() has already occurred, so cut vertices
         //       will have external face edges other than the first and last arcs
         //       Hence we need this more sophisticated traversal method
-        Jin = gp_GetTwinArc(theGraph, Jout);
+        eTwin = gp_GetTwinArc(theGraph, e);
 
         // Now we get the next arc in rotation order as the new arc out to the
         // vertex after nextVertex.  This sets us up for the next iteration.
@@ -371,7 +334,7 @@ void _MarkExternalFaceVertices(graphP theGraph, int startVertex)
         //       and last arcs.  We will visit those vertices multiple times,
         //       which is OK (just that we have to explain why we're calculating
         //       jout in this way).
-        Jout = gp_GetNextArcCircular(theGraph, Jin);
+        e = gp_GetNextArcCircular(theGraph, eTwin);
 
         // Now things get really interesting.  The DFS root (startVertex) may
         // itself be a cut vertex to which multiple bicomps have been joined.
@@ -384,7 +347,7 @@ void _MarkExternalFaceVertices(graphP theGraph, int startVertex)
         // of startVertex, we need to proceed until we reenter startVertex
         // using its last arc.
 
-    } while (Jin != gp_GetLastArc(theGraph, startVertex));
+    } while (eTwin != gp_GetLastArc(theGraph, startVertex));
 }
 
 
@@ -442,19 +405,19 @@ int _CheckObstructionIntegrity(graphP theGraph, graphP origGraph)
 int  _getImageVertices(graphP theGraph, int *degrees, int maxDegree,
                        int *imageVerts, int maxNumImageVerts)
 {
-int I, imageVertPos, degree;
+int K, v, imageVertPos, degree;
 
-     for (I = 0; I <= maxDegree; I++)
-          degrees[I] = 0;
+     for (degree = 0; degree <= maxDegree; degree++)
+          degrees[degree] = 0;
 
-     for (I = 0; I < maxNumImageVerts; I++)
-          imageVerts[I] = NIL;
+     for (K = 0; K < maxNumImageVerts; K++)
+          imageVerts[K] = NIL;
 
      imageVertPos = 0;
 
-     for (I = 0; I < theGraph->N; I++)
+     for (v = gp_GetFirstVertex(theGraph); gp_VertexInRange(theGraph, v); v++)
      {
-          degree = gp_GetVertexDegree(theGraph, I);
+          degree = gp_GetVertexDegree(theGraph, v);
           if (degree == 1)
               return NOTOK;
           if (degree > maxDegree)
@@ -463,7 +426,7 @@ int I, imageVertPos, degree;
           degrees[degree]++;
 
           if (imageVertPos < maxNumImageVerts && degree > 2)
-              imageVerts[imageVertPos++] = I;
+              imageVerts[imageVertPos++] = v;
           else if (degree > 2)
               return NOTOK;
      }
@@ -496,7 +459,7 @@ int I, imageVertPos, degree;
 int _TestForCompleteGraphObstruction(graphP theGraph, int numVerts,
                                      int *degrees, int *imageVerts)
 {
-    int I, J;
+    int v, w;
 
     // We need to make sure we have numVerts vertices of degree numVerts-1
     // For example, if numVerts==5, then we're looking for a K5, so we
@@ -509,25 +472,24 @@ int _TestForCompleteGraphObstruction(graphP theGraph, int numVerts,
         return FALSE;
 
     // We clear all the vertex visited flags
-    for (I = 0; I < theGraph->N; I++)
-        theGraph->G[I].visited = 0;
+    _ClearVertexVisitedFlags(theGraph, FALSE);
 
     // For each pair of image vertices, we test that there is a path
     // between the two vertices.  If so, the visited flags of the
     // internal vertices along the path are marked
     //
-    for (I = 0; I < numVerts; I++)
-        for (J = 0; J < numVerts; J++)
-           if (I != J)
-               if (_TestPath(theGraph, imageVerts[I],
-                                       imageVerts[J]) != TRUE)
+    for (v = 0; v < numVerts; v++)
+        for (w = 0; w < numVerts; w++)
+           if (v != w)
+               if (_TestPath(theGraph, imageVerts[v],
+                                       imageVerts[w]) != TRUE)
                    return FALSE;
 
     // The visited flags should have marked only degree two vertices,
     // so for every marked vertex, we subtract one from the count of
     // the degree two vertices.
-    for (I = 0; I < theGraph->N; I++)
-        if (theGraph->G[I].visited)
+    for (v = gp_GetFirstVertex(theGraph); gp_VertexInRange(theGraph, v); v++)
+        if (gp_GetVertexVisited(theGraph, v))
             degrees[2]--;
 
     /* If every degree 2 vertex is used in a path between image
@@ -555,7 +517,7 @@ int _TestForCompleteGraphObstruction(graphP theGraph, int numVerts,
 
 int _TestForK33GraphObstruction(graphP theGraph, int *degrees, int *imageVerts)
 {
-int  I, imageVertPos, temp, success;
+int  v, K, imageVertPos, temp, success;
 
 	if (degrees[4] != 0)
 		return FALSE;
@@ -568,7 +530,8 @@ int  I, imageVertPos, temp, success;
 
      for (imageVertPos = 3; imageVertPos < 6; imageVertPos++)
      {
-          I = success = 0;
+          K = 0;
+          success = FALSE;
           do {
              if (_TestPath(theGraph, imageVerts[imageVertPos], imageVerts[0]) == TRUE)
              {
@@ -576,11 +539,11 @@ int  I, imageVertPos, temp, success;
                  break;
              }
 
-             I++;
-             temp = imageVerts[I];
-             imageVerts[I] = imageVerts[imageVertPos];
+             K++;
+             temp = imageVerts[K];
+             imageVerts[K] = imageVerts[imageVertPos];
              imageVerts[imageVertPos] = temp;
-          }  while (I < 3);
+          }  while (K < 3);
 
           if (!success)
               return FALSE;
@@ -589,17 +552,16 @@ int  I, imageVertPos, temp, success;
      /* Now test the paths between each of the first three vertices and
             each of the last three vertices */
 
-     for (I = 0; I < theGraph->N; I++)
-          theGraph->G[I].visited = 0;
+     _ClearVertexVisitedFlags(theGraph, FALSE);
 
      for (imageVertPos=0; imageVertPos<3; imageVertPos++)
-          for (I=3; I<6; I++)
+          for (K=3; K<6; K++)
                if (_TestPath(theGraph, imageVerts[imageVertPos],
-                                       imageVerts[I]) != TRUE)
+                                       imageVerts[K]) != TRUE)
                    return FALSE;
 
-     for (I = 0; I < theGraph->N; I++)
-          if (theGraph->G[I].visited)
+     for (v = gp_GetFirstVertex(theGraph); gp_VertexInRange(theGraph, v); v++)
+          if (gp_GetVertexVisited(theGraph, v))
               degrees[2]--;
 
      /* If every degree 2 vertex is used in a path between image
@@ -678,7 +640,7 @@ int  degrees[5], imageVerts[6];
 
 int _TestForK23GraphObstruction(graphP theGraph, int *degrees, int *imageVerts)
 {
-int  I, J, imageVertPos;
+int  v, e, imageVertPos;
 
      // This function operates over the imageVerts results produced by
      // getImageVertices, which only finds vertices of degree 3 or higher.
@@ -701,14 +663,14 @@ int  I, J, imageVertPos;
      // the two degree 3 image vertices are in the same partition
      // and hence must not be adjacent.
 
-     J = gp_GetFirstArc(theGraph, imageVerts[0]);
-     while (gp_IsArc(theGraph, J))
+     e = gp_GetFirstArc(theGraph, imageVerts[0]);
+     while (gp_IsArc(e))
      {
-         imageVerts[imageVertPos] = theGraph->G[J].v;
+         imageVerts[imageVertPos] = gp_GetNeighbor(theGraph, e);
          if (imageVerts[imageVertPos] == imageVerts[1])
              return FALSE;
          imageVertPos++;
-         J = gp_GetNextArc(theGraph, J);
+         e = gp_GetNextArc(theGraph, e);
      }
 
      /* The paths from imageVerts[0] to each of the new degree 2
@@ -716,19 +678,19 @@ int  I, J, imageVertPos;
           Now test the paths between each of the degree 2 image
           vertices and imageVerts[1]. */
 
-     for (I = 0; I < theGraph->N; I++)
-          theGraph->G[I].visited = 0;
+     _ClearVertexVisitedFlags(theGraph, FALSE);
 
      for (imageVertPos=2; imageVertPos<5; imageVertPos++)
      {
           if (_TestPath(theGraph, imageVerts[imageVertPos],
                                   imageVerts[1]) != TRUE)
-                   return FALSE;
-          theGraph->G[imageVerts[imageVertPos]].visited = 1;
+              return FALSE;
+
+          gp_SetVertexVisited(theGraph, imageVerts[imageVertPos]);
      }
 
-     for (I = 0; I < theGraph->N; I++)
-          if (theGraph->G[I].visited)
+     for (v = gp_GetFirstVertex(theGraph); gp_VertexInRange(theGraph, v); v++)
+          if (gp_GetVertexVisited(theGraph, v))
               degrees[2]--;
 
      /* If every degree 2 vertex is used in a path between the
@@ -803,18 +765,17 @@ int  degrees[4], imageVerts[5];
 
 int  _TestPath(graphP theGraph, int U, int V)
 {
-int  J;
+	 int  e = gp_GetFirstArc(theGraph, U);
 
-     J = gp_GetFirstArc(theGraph, U);
-     while (gp_IsArc(theGraph, J))
+     while (gp_IsArc(e))
      {
-         if (_TryPath(theGraph, J, V) == OK)
+         if (_TryPath(theGraph, e, V) == OK)
          {
-             _MarkPath(theGraph, J);
+             _MarkPath(theGraph, e);
              return TRUE;
          }
 
-         J = gp_GetNextArc(theGraph, J);
+         e = gp_GetNextArc(theGraph, e);
      }
 
      return FALSE;
@@ -831,23 +792,23 @@ int  J;
  boolean false otherwise.
  ********************************************************************/
 
-int  _TryPath(graphP theGraph, int J, int V)
+int  _TryPath(graphP theGraph, int e, int V)
 {
-int  Jin, nextVertex;
+int  eTwin, nextVertex;
 
-     nextVertex = theGraph->G[J].v;
+     nextVertex = gp_GetNeighbor(theGraph, e);
 
      // while nextVertex is strictly degree 2
-     while (gp_IsArc(theGraph, gp_GetFirstArc(theGraph, nextVertex)) &&
-    		gp_IsArc(theGraph, gp_GetLastArc(theGraph, nextVertex)) &&
+     while (gp_IsArc(gp_GetFirstArc(theGraph, nextVertex)) &&
+    		gp_IsArc(gp_GetLastArc(theGraph, nextVertex)) &&
     		gp_GetNextArc(theGraph, gp_GetFirstArc(theGraph, nextVertex)) == gp_GetLastArc(theGraph, nextVertex))
      {
-         Jin = gp_GetTwinArc(theGraph, J);
-         J = gp_GetFirstArc(theGraph, nextVertex);
-         if (J == Jin)
-             J = gp_GetLastArc(theGraph, nextVertex);
+    	 eTwin = gp_GetTwinArc(theGraph, e);
+         e = gp_GetFirstArc(theGraph, nextVertex);
+         if (e == eTwin)
+             e = gp_GetLastArc(theGraph, nextVertex);
 
-         nextVertex = theGraph->G[J].v;
+         nextVertex = gp_GetNeighbor(theGraph, e);
      }
 
      return nextVertex == V ? TRUE : FALSE;
@@ -861,24 +822,24 @@ int  Jin, nextVertex;
  a starting vertex U.
  ********************************************************************/
 
-void _MarkPath(graphP theGraph, int J)
+void _MarkPath(graphP theGraph, int e)
 {
-int  Jin, nextVertex;
+int  eTwin, nextVertex;
 
-     nextVertex = theGraph->G[J].v;
+     nextVertex = gp_GetNeighbor(theGraph, e);
      // while nextVertex is strictly degree 2
-     while (gp_IsArc(theGraph, gp_GetFirstArc(theGraph, nextVertex)) &&
-    		gp_IsArc(theGraph, gp_GetLastArc(theGraph, nextVertex)) &&
+     while (gp_IsArc(gp_GetFirstArc(theGraph, nextVertex)) &&
+    		gp_IsArc(gp_GetLastArc(theGraph, nextVertex)) &&
     		gp_GetNextArc(theGraph, gp_GetFirstArc(theGraph, nextVertex)) == gp_GetLastArc(theGraph, nextVertex))
      {
-         theGraph->G[nextVertex].visited = 1;
+         gp_SetVertexVisited(theGraph, nextVertex);
 
-         Jin = gp_GetTwinArc(theGraph, J);
-         J = gp_GetFirstArc(theGraph, nextVertex);
-         if (J == Jin)
-             J = gp_GetLastArc(theGraph, nextVertex);
+         eTwin = gp_GetTwinArc(theGraph, e);
+         e = gp_GetFirstArc(theGraph, nextVertex);
+         if (e == eTwin)
+             e = gp_GetLastArc(theGraph, nextVertex);
 
-         nextVertex = theGraph->G[J].v;
+         nextVertex = gp_GetNeighbor(theGraph, e);
      }
 }
 
@@ -898,7 +859,7 @@ int  Jin, nextVertex;
 
 int  _TestSubgraph(graphP theSubgraph, graphP theGraph)
 {
-int I, J;
+int v, e, degreeCount;
 int Result = TRUE;
 int invokeSortOnGraph = FALSE;
 int invokeSortOnSubgraph = FALSE;
@@ -923,61 +884,61 @@ int invokeSortOnSubgraph = FALSE;
 
 /* We clear all visitation flags */
 
-     for (I = 0; I < theGraph->N; I++)
-          theGraph->G[I].visited = 0;
+     _ClearVertexVisitedFlags(theGraph, FALSE);
 
 /* For each vertex... */
-
-     for (I = 0; I < theSubgraph->N; I++)
+     for (v = gp_GetFirstVertex(theSubgraph), degreeCount = 0; gp_VertexInRange(theSubgraph, v); v++)
      {
-          /* For each neighbor w in the adjacency list of vertex I in the
+          /* For each neighbor w in the adjacency list of vertex v in the
                 subgraph, set the visited flag in w in the graph */
 
-          J = gp_GetFirstArc(theSubgraph, I);
-          while (gp_IsArc(theSubgraph, J))
+          e = gp_GetFirstArc(theSubgraph, v);
+          while (gp_IsArc(e))
           {
-        	  if (theSubgraph->G[J].v == NIL)
+        	  if (gp_IsNotVertex(gp_GetNeighbor(theSubgraph, e)))
         	  {
         		  Result = FALSE;
         		  break;
         	  }
-              theGraph->G[theSubgraph->G[J].v].visited = 1;
-              J = gp_GetNextArc(theSubgraph, J);
+        	  degreeCount++;
+        	  gp_SetVertexVisited(theGraph, gp_GetNeighbor(theSubgraph, e));
+              e = gp_GetNextArc(theSubgraph, e);
           }
 
           if (Result != TRUE)
         	  break;
 
-          /* For each neighbor w in the adjacency list of vertex I in the graph,
+          /* For each neighbor w in the adjacency list of vertex v in the graph,
                 clear the visited flag in w in the graph */
 
-          J = gp_GetFirstArc(theGraph, I);
-          while (gp_IsArc(theGraph, J))
+          e = gp_GetFirstArc(theGraph, v);
+          while (gp_IsArc(e))
           {
-        	  if (theGraph->G[J].v == NIL)
+        	  if (gp_IsNotVertex(gp_GetNeighbor(theGraph, e)))
         	  {
         		  Result = FALSE;
         		  break;
         	  }
-              theGraph->G[theGraph->G[J].v].visited = 0;
-              J = gp_GetNextArc(theGraph, J);
+        	  gp_ClearVertexVisited(theGraph, gp_GetNeighbor(theGraph, e));
+              e = gp_GetNextArc(theGraph, e);
           }
 
           if (Result != TRUE)
         	  break;
 
-          /* For each neighbor w in the adjacency list of vertex I in the
-                subgraph, set the visited flag in w in the graph */
+          /* For each neighbor w in the adjacency list of vertex v in the subgraph,
+             ensure that the visited flag in w was cleared (otherwise, the "subgraph"
+             would incorrectly contain an adjacency not contained in the ("super") graph) */
 
-          J = gp_GetFirstArc(theSubgraph, I);
-          while (gp_IsArc(theSubgraph, J))
+          e = gp_GetFirstArc(theSubgraph, v);
+          while (gp_IsArc(e))
           {
-              if (theGraph->G[theSubgraph->G[J].v].visited)
+              if (gp_GetVertexVisited(theGraph, gp_GetNeighbor(theSubgraph, e)))
               {
             	  Result = FALSE;
             	  break;
               }
-              J = gp_GetNextArc(theSubgraph, J);
+              e = gp_GetNextArc(theSubgraph, e);
           }
 
           if (Result != TRUE)
@@ -990,5 +951,15 @@ int invokeSortOnSubgraph = FALSE;
     if (invokeSortOnGraph)
         gp_SortVertices(theGraph);
 
+    // Assuming theSubgraph is a subgraph, we also do an extra integrity check to ensure
+    // proper edge array utilization
+    if (Result == TRUE)
+    {
+    	// If the edge count is wrong, we fail the subgraph test in a way that invokes
+    	// the name NOTOK so that in debug mode there is more trace on the failure.
+    	if (degreeCount != 2*theSubgraph->M)
+    		Result = NOTOK == FALSE ? NOTOK : FALSE;
+    }
+
      return Result;
 }
